#!/usr/bin/env python
# -*- coding: utf-8 -*-

from redis import StrictRedis

from redylitics.sentinels import SentinelConnection


class BaseClient(object):
    """a client interface
    """

    def get_one(self, key):
        """gets the value of a single key

        :param key: the key to retrieve
        :type key: str

        :return: the key's value
        :rtype: int
        """
        raise NotImplementedError

    def get_many(self, keys):
        """gets the values of many keys

        :param keys: a list of keys
        :type keys: list

        :return: a list of key-value tuples
        :rtype: list
        """
        raise NotImplementedError

    def get_one_hash(self, key, subkeys=None):
        """gets a single hash object, optionally filtered to a key

        :param key: the name of the key
        :type key: str

        :param subkeys: a list of hash keys
        :type subkeys: list

        :return: the requested object
        :rtype: dict
        """
        raise NotImplementedError

    def set_one(self, key, value):
        """increments a key by a given value

        :param key: the name to update
        :type key: str

        :param value: the value to increment by
        :type value: int

        :return: the updated value
        :rtype: int
        """
        raise NotImplementedError

    def set_one_hash(self, key, subkey, value):
        """increments a hash key by a given value

        :param key: the name of the hash
        :type key: str

        :param subkey: the name of the hash key to update
        :type subkey: str

        :param value: the value to increment by
        :type value: int

        :return: the updated value
        :rtype: int
        """
        raise NotImplementedError


class SentinelClient(BaseClient):
    """a redis client that holds a sentinel connection and uses master to write and slaves to read
    """

    def __init__(self, sentinel_hosts, sentinel_port=26379, master_name="mymaster"):
        """initializes a sentinel connection

        :param sentinel_hosts: a list of ip addresses
        :type sentinel_hosts: list

        :param sentinel_port: the port number to connect to the sentinels
        :type sentinel_port: int

        :param master_name: the name of the master database
        :type master_name: str
        """
        self.sentinel = SentinelConnection(sentinel_hosts, sentinel_port, master_name)

    def get_one(self, key):
        client = self.sentinel.get_slave()
        return int(client.get(key))

    def get_many(self, keys):
        client = self.sentinel.get_slave()
        keys = sorted(client.keys(keys))
        values = client.mget(keys)
        for index, value in enumerate(values):
            values[index] = int(value)
        return zip(keys, values)

    def get_one_hash(self, key, subkeys=None):
        client = self.sentinel.get_slave()
        if subkeys:
            _res = client.hmget(key, subkeys)
            res = {}
            for index, key in enumerate(subkeys):
                res[key] = int(_res[index])
        else:
            res = client.hgetall(key)
            for key, value in res.items():
                res[key] = int(value)
        return res

    def set_one(self, key, value):
        client = self.sentinel.get_master()
        return client.incrby(key, value)

    def set_one_hash(self, key, subkey, value):
        client = self.sentinel.get_master()
        return client.hincrby(key, subkey, value)


class StrictClient(BaseClient):
    """a wrapper around the redis.StrictRedis object
    """

    def __init__(self, **kwargs):
        """initializes the client object with a `.client` attribute

        :param kwargs: keyword arguments for building a `redis.StrictRedis` client
        :type kwargs: dict
        """
        self.client = StrictRedis(**kwargs)

    def get_one(self, key):
        return self.client.get(key)

    def get_many(self, keys):
        keys = self.client.keys(keys)
        values = self.client.mget(keys)
        return zip(keys, values)

    def get_one_hash(self, key, subkeys=None):
        if subkeys:
            return self.client.hmget(key, subkeys)
        return self.client.hgetall(key)

    def set_one(self, key, value):
        return self.client.incrby(key, value)

    def set_one_hash(self, key, subkey, value):
        return self.client.hincrby(key, subkey, value)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

from redylitics.defaults import *


# sentinels stuff

_SENTINEL_HOSTS = os.environ.get("RA_SENTINEL_HOSTS", DEF_SENTINEL_HOSTS)
SENTINEL_HOSTS = [h.strip() for h in _SENTINEL_HOSTS.split(",")]
SENTINEL_PORT = os.environ.get("RA_SENTINEL_PORT", DEF_SENTINEL_PORT)
SENTINEL_MASTER_NAME = os.environ.get("RA_SENTINEL_MASTER_NAME", DEF_SENTINEL_MASTER_NAME)


# strict clients stuff

REDIS_HOST = os.environ.get("RA_REDIS_HOST", DEF_REDIS_HOST)
REDIS_PORT = os.environ.get("RA_REDIS_PORT", DEF_REDIS_PORT)
REDIS_DB = os.environ.get("RA_REDIS_DB", DEF_REDIS_PORT)


# pageview stuff

WRITE_CHANNEL_REGEX_PATTERN = os.environ.get("RA_WRITE_CHANNEL_REGEX", DEF_WRITE_CHANNEL_REGEX_PATTERN)
WRITE_DATE_REGEX_PATTERN = os.environ.get("RA_WRITE_DATE_REGEX", DEF_WRITE_DATE_REGEX_PATTERN)
READ_CHANNEL_REGEX_PATTERN = os.environ.get("RA_READ_CHANNEL_REGEX", DEF_READ_CHANNEL_REGEX_PATTERN)
READ_DATE_REGEX_PATTERN = os.environ.get("RA_READ_DATE_REGEX", DEF_READ_DATE_REGEX_PATTERN)
DATE_EXTRACTION_REGEX_PATTERN = os.environ.get("RA_DATE_EXTRACTION_REGEX", DEF_DATE_EXTRACTION_REGEX_PATTERN)


# video stuff

VIDEO_PREFIX = os.environ.get("RA_VIDEO_PREFIX", DEF_VIDEO_PREFIX)
VIDEO_ID_REGEX_PATTERN = os.environ.get("RA_VIDEO_ID_REGEX", DEF_VIDEO_ID_REGEX_PATTERN)
VIDEO_EVENT_REGEX_PATTERN = os.environ.get("RA_VIDEO_EVENT_REGEX", DEF_VIDEO_EVENT_REGEX_PATTERN)


# video ad stuff

VIDEO_AD_PREFIX = os.environ.get("RA_VIDEO_AD_PREFIX", DEF_VIDEO_AD_PREFIX)
VIDEO_AD_ID_REGEX_PATTERN = os.environ.get("RA_VIDEO_AD_ID_REGEX", DEF_VIDEO_AD_ID_REGEX_PATTERN)
VIDEO_AD_EVENT_REGEX_PATTERN = os.environ.get("RA_VIDEO_AD_EVENT_REGEX", DEF_VIDEO_AD_EVENT_REGEX_PATTERN)


# wsgi stuff

REFERER_ANCHORED_REGEX_PATTERN = os.environ.get("RA_REFERER_ANCHORED_REGEX", DEF_REFERER_ANCHORED_REGEX_PATTERN)
REFERER_EXTRACT_REGEX_PATTERN = os.environ.get("RA_REFERER_EXTRACT_REGEX", DEF_REFERER_EXTRACT_REGEX_PATTERN)
FLUSH_INTERVAL = os.environ.get("RA_FLUSH_INTERVAL", DEF_FLUSH_INTERVAL)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from collections import Counter
import json
import re

import gevent
from gevent import queue

from redylitics.config import FLUSH_INTERVAL, REFERER_EXTRACT_REGEX_PATTERN, REFERER_ANCHORED_REGEX_PATTERN
from redylitics.endpoints import update_channels, update_video_ads, update_videos, \
    read_channels, read_videos, read_video_ads
from redylitics.utils import logger, gif

try:
    from urllib.parse import parse_qs
except ImportError:
    from urlparse import parse_qs


# init the gevent queue
channel_queue = queue.Queue()
video_queue = queue.Queue()
video_ad_queue = queue.Queue()
flush_interval = FLUSH_INTERVAL

# endpoint regexes
referer_extract_regex = re.compile(REFERER_EXTRACT_REGEX_PATTERN)
referer_anchored_regex = re.compile(REFERER_ANCHORED_REGEX_PATTERN)


# get http referer
def get_referer(env):
    try:
        ref = env["HTTP_REFERER"]
    except KeyError:
        return None
    if referer_anchored_regex.search(ref):
        matches = referer_extract_regex.findall(ref)
        if len(matches):
            return matches[0]
    return None


# update page views
def channel_counts():
    while 1:
        gevent.sleep(flush_interval)
        channels = Counter()
        while 1:
            try:
                channel = channel_queue.get_nowait()
                channels[channel] += 1
            except queue.Empty:
                break
            except Exception as e:
                logger.exception(e)
                break
        if len(channels):
            gevent.spawn(update_channels, channels)


# update videos
def video_counts():
    while 1:
        gevent.sleep(flush_interval)
        videos = Counter()
        while 1:
            try:
                video = video_queue.get_nowait()
                videos[video] += 1
            except queue.Empty:
                break
            except Exception as e:
                logger.exception(e)
                break
        if len(videos):
            gevent.spawn(update_videos, videos)


# update video ads
def video_ad_counts():
    while 1:
        gevent.sleep(flush_interval)
        video_ads = Counter()
        while 1:
            try:
                ad = video_ad_queue.get_nowait()
                video_ads[ad] += 1
            except queue.Empty:
                break
            except Exception as e:
                logger.exception(e)
                break
        if len(video_ads):
            gevent.spawn(update_video_ads, video_ads)


# create the wait loops
gevent.spawn(channel_counts)
gevent.spawn(video_counts)
gevent.spawn(video_ad_counts)


# main loop
def application(env, start_response):
    # get the request path
    path = env["PATH_INFO"].lower()

    # get the query params
    try:
        params = parse_qs(env["QUERY_STRING"])
    except KeyError:
        params = None

    # get the http referer header
    referer = get_referer(env)

    # update page views
    if path == "/pageview.gif" and referer:
        start_response("200 OK", [("Content-Type", "image/gif")])
        yield gif
        channel_queue.put(referer)

    # update videos
    elif path == "video.gif" and referer and params:
        start_response("200 OK", [("Content-Type", "image/gif")])
        yield gif
        video = params.get("video", [None])[0]
        event = params.get("event", [None])[0]
        video_queue.put((video, event))

    # update video ads
    elif path == "/video-ad.gif" and referer and params:
        start_response("200 OK", [("Content-Type", "image/gif")])
        yield gif
        video = params.get("video_ad", [None])[0]
        event = params.get("event", [None])[0]
        video_ad_queue.put((video, event))

    # read page views
    elif path == "/pageview.json" and params:
        channel = params.get("channel", [None])[0]
        res, ok = read_channels(channel)
        if ok:
            start_response("200 OK", [("Content-Type", "application/json")])
        else:
            start_response("400 Bad Request", [("Content-Type", "application/json")])
        yield json.dumps(res)

    # read videos
    elif path == "/video.json" and params:
        video = params.get("video", [None])[0]
        event = params.get("event", [None])[0]
        res, ok = read_videos(video, event)
        if ok:
            start_response("200 OK", [("Content-Type", "application/json")])
        else:
            start_response("400 Bad Request", [("Content-Type", "application/json")])
        yield json.dumps(res)

    # read video ads
    elif path == "/video-ad.json" and params:
        video = params.get("video_ad", [None])[0]
        event = params.get("event", [None])[0]
        res, ok = read_video_ads(video, event)
        if ok:
            start_response("200 OK", [("Content-Type", "application/json")])
        else:
            start_response("400 Bad Request", [("Content-Type", "application/json")])
        yield json.dumps(res)

    # bologna
    else:
        start_response("403 Forbidden", [("Content-Type", "text/plain")])
        yield "(\/) (°,,,°) (\/)"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from datetime import date, datetime
import re

from redylitics.clients import SentinelClient
from redylitics.config import *
from redylitics.utils import logger


# pageview stuff

write_channel_regex = re.compile(WRITE_CHANNEL_REGEX_PATTERN)
write_date_regex = re.compile(WRITE_DATE_REGEX_PATTERN)
read_channel_regex = re.compile(READ_CHANNEL_REGEX_PATTERN)
read_date_regex = re.compile(READ_DATE_REGEX_PATTERN)
date_extraction_regex = re.compile(DATE_EXTRACTION_REGEX_PATTERN)


# video stuff

video_prefix = VIDEO_PREFIX
video_id_regex = re.compile(VIDEO_ID_REGEX_PATTERN)
video_event_regex = re.compile(VIDEO_EVENT_REGEX_PATTERN)


# video ad stuff

video_ad_prefix = VIDEO_AD_PREFIX
video_ad_id_regex = re.compile(VIDEO_AD_ID_REGEX_PATTERN)
video_ad_event_regex = re.compile(VIDEO_AD_EVENT_REGEX_PATTERN)


def validate_date(date_string):
    """validates a date string as being either YYYYmmdd or YYYYmm*

    :param date_string: the date string to be validated
    :type date_string: str

    :return: is the date valid?
    :rtype: bool
    """
    try:
        _ = datetime.strptime(date_string, '%Y%m%d')
        return True
    except (TypeError, ValueError):
        try:
            _ = datetime.strptime(date_string, '%Y%m*')
            return True
        except (TypeError, ValueError):
            return False


def update_channels(counter):
    """attempts to update a channel's pageview

    :param counter: the pageviews counter from the wsgi process
    :type counter: collections.Counter
    """
    t = date.today()
    today = datetime(t.year, t.month, t.day).strftime("%Y%m%d")

    for channel, incr in counter.items():
        channel = "{}{}".format(channel, today)

        channel_is_valid = write_channel_regex.match(channel) is not None
        if not channel_is_valid:
            logger.error("bad channel. skipping. `{}`".format(channel))
            continue

        incr_is_valid = isinstance(incr, int)
        if not incr_is_valid:
            logger.error("bad increment value. skipping channel {}. `{}`".format(channel, incr))
            continue

        try:
            client = SentinelClient(SENTINEL_HOSTS, SENTINEL_PORT, SENTINEL_MASTER_NAME)
            output = client.set_one(channel, incr)
            logger.info("incr {} {}: {}".format(channel, incr, output))
        except Exception as e:
            logger.exception(e)


def update_videos(counter):
    """attempts to update a video's event counts

    :param counter: the video counter from the wsgi process
    :type counter: collections.Counter
    """
    for (video, event), incr in counter.items():
        video_is_valid = video_id_regex.match(video) is not None
        if not video_is_valid:
            logger.error("bad video id. skipping. `{}`".format(video))
            continue

        video = "{}.{}".format(video_prefix, video)

        event_is_valid = video_event_regex.match(event) is not None
        if not event_is_valid:
            logger.error("bad event. skipping video {}. `{}`".format(video, event))
            continue

        incr_is_valid = isinstance(incr, int)
        if not incr_is_valid:
            logger.error("bad increment value. skipping video.event {}.{}. `{}`".format(video, event, incr))
            continue

        try:
            client = SentinelClient(SENTINEL_HOSTS, SENTINEL_PORT, SENTINEL_MASTER_NAME)
            output = client.set_one_hash(video, event, incr)
            logger.info("hincr {} {} {}: {}".format(video, event, incr, output))
        except Exception as e:
            logger.exception(e)


def update_video_ads(counter):
    """attempts to update a video ad's event counts

    :param counter: the video ad counter from the wsgi process
    :type counter: collections.Counter
    """
    for (video, event), incr in counter.items():
        video_is_valid = video_ad_id_regex.match(video) is not None
        if not video_is_valid:
            logger.error("bad video ad id. skipping. `{}`".format(video))
            continue

        video = "{}.{}".format(video_ad_prefix, video)

        event_is_valid = video_ad_event_regex.match(event) is not None
        if not event_is_valid:
            logger.error("bad event. skipping video ad {}. `{}`".format(video, event))
            continue

        incr_is_valid = isinstance(incr, int)
        if not incr_is_valid:
            logger.error("bad increment value. skipping video_ad.event {}.{}. `{}`".format(video, event, incr))
            continue

        try:
            client = SentinelClient(SENTINEL_HOSTS, SENTINEL_PORT, SENTINEL_MASTER_NAME)
            output = client.set_one_hash(video, event, incr)
            logger.info("hincr {} {} {}: {}".format(video, event, incr, output))
        except Exception as e:
            logger.exception(e)


def read_channels(channel):
    """gets the dates and pageviews for a given channel + date string

    :param channel: the name of the channel + a date string
    :type channel: str

    :return: date: value key pairs and an is ok? value
    :rtype: dict, bool
    """
    channel_is_valid = read_channel_regex.match(channel) is not None
    if not channel_is_valid:
        return {"error": "The channel name supplied is not valid"}, False

    date_string = read_date_regex.findall(channel)
    if not len(date_string):
        return {"error": "Could not parse date attached to the channel: {}".format(read_date_regex.pattern)}, False
    date_string = date_string[0]

    date_is_valid = validate_date(date_string)
    if not date_is_valid:
        return {"error": "The date attached to the channel is not valid"}, False

    try:
        client = SentinelClient(SENTINEL_HOSTS, SENTINEL_PORT, SENTINEL_MASTER_NAME)
        result = client.get_many(channel)

        keys = [r[0] for r in result]
        values = [r[1] for r in result]

        for index, value in enumerate(values):
            values[index] = int(value)

        dates = []
        for key in keys:
            matches = date_extraction_regex.findall(key)
            if len(matches):
                date_key = datetime.strptime(matches[0], "%Y%m%d").strftime("%Y-%m-%d")
                dates.append(date_key)

        return dict(zip(dates, values)), True

    except Exception as e:
        return str(e), False


def read_videos(video, event=None):
    """gets the counts for events for a given video

    :param video: the id of the video
    :type video: str

    :param event: the name of the event
    :type event: str

    :return: event: value key pairs and an is ok? value
    :rtype: dict, bool
    """
    video_is_valid = video_id_regex.match(video) is not None
    if not video_is_valid:
        return {"error": "The video id supplied is not valid"}, False

    video = "{}.{}".format(video_prefix, video)

    if event:
        event_is_valid = video_event_regex.match(event) is not None
        if not event_is_valid:
            return {"error": "The event name supplied is not valid"}, False
        event = [event, ]

    try:
        client = SentinelClient(SENTINEL_HOSTS, SENTINEL_PORT, SENTINEL_MASTER_NAME)
        result = client.get_one_hash(video, event)
        return result, True
    except Exception as e:
        return str(e), False


def read_video_ads(ad, event=None):
    """gets the counts for events for a given video ad

    :param ad: the id of the video ad
    :type ad: str

    :param event: the name of the event
    :type event: str

    :return: event: value key pairs and an is ok? value
    :rtype: dict, bool
    """
    video_is_valid = video_id_regex.match(ad) is not None
    if not video_is_valid:
        return {"error": "The video id supplied is not valid"}, False

    video = "{}.{}".format(video_ad_prefix, ad)

    if event:
        event_is_valid = video_event_regex.match(event) is not None
        if not event_is_valid:
            return {"error": "The event name supplied is not valid"}, False
        event = [event, ]

    try:
        client = SentinelClient(SENTINEL_HOSTS, SENTINEL_PORT, SENTINEL_MASTER_NAME)
        result = client.get_one_hash(video, event)
        return result, True
    except Exception as e:
        return str(e), False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from collections import Counter
from datetime import date, datetime
from unittest import TestCase

from .clients import SentinelClient
from .config import *
from .endpoints import update_channels, update_videos, update_video_ads, read_channels, read_videos, read_video_ads
from .wsgi import get_referer


class SentinelClientTests(TestCase):
    def setUp(self):
        self.client = SentinelClient(SENTINEL_HOSTS, SENTINEL_PORT, SENTINEL_MASTER_NAME)

    def tearDown(self):
        self.client.sentinel.get_master().flushdb()

    def test_set(self):
        result = self.client.set_one("test", 1)
        self.assertEqual(result, 1)

    def test_set_hash(self):
        result = self.client.set_one_hash("test-hash", "test", 1)
        self.assertEqual(result, 1)

    def test_get_one(self):
        _ = self.client.set_one("test", 1)
        result = self.client.get_one("test")
        self.assertEqual(result, 1)

    def test_get_many(self):
        _ = self.client.set_one("multi1", 1)
        _ = self.client.set_one("multi2", 1)
        _ = self.client.set_one("multi3", 1)
        result = self.client.get_many("multi*")
        self.assertEqual(result, [("multi1", 1), ("multi2", 1), ("multi3", 1)])

    def test_get_one_hash(self):
        _ = self.client.set_one_hash("test-hash", "test", 1)
        result = self.client.get_one_hash("test-hash")
        self.assertEqual(result, {"test": 1})
        result = self.client.get_one_hash("test-hash", ["test", ])
        self.assertEqual(result, {"test": 1})


class RefererTests(TestCase):
    def test_get_referer(self):
        good_referers = (
            "http://www.theonion.com/articles/blah-123",
            "http://www.avclub.com/videos/blah-123",
            "http://www.clickhole.com/quizzes/blah-123",
            "http://www.onionstudios.com/channels/the-onion",
        )
        bad_referers = (
            "http://barf.theonion.com/articles/blah-123",
            "http://puke.avclub.com/videos/blah-123",
            "http://hurl.clickhole.com/quizzes/blah-123",
            "http://yuck.onionstudios.com/channels/the-onion",
            "http://www.barftheonion.com/articles/blah-123",
            "http://www.pukeavclub.com/videos/blah-123",
            "http://www.hurlclickhole.com/quizzes/blah-123",
            "http://www.yuckonionstudios.com/channels/the-onion",
            "http://theonion.local/articles/blah-123",
            "http://avclub.local/videos/blah-123",
            "http://clickhole.local/quizzes/blah-123",
            "http://onionstudios.local/channels/the-onion",
        )
        for ref in good_referers:
            self.assertIsNotNone(get_referer({"HTTP_REFERER": ref}))
        for ref in bad_referers:
            self.assertIsNone(get_referer({"HTTP_REFERER": ref}))


class ReadChannelsEndpointTests(TestCase):
    def setUp(self):
        self.client = SentinelClient(SENTINEL_HOSTS, SENTINEL_PORT, SENTINEL_MASTER_NAME)

    def tearDown(self):
        self.client.sentinel.get_master().flushdb()

    def test_read_channels(self):
        key = "theonion20170101"
        _ = self.client.set_one(key, 100)

        data = self.client.get_one(key)
        self.assertEqual(data, 100)

        res, ok = read_channels(key)
        print(res)
        self.assertTrue(ok)
        self.assertEqual(res, {"2017-01-01": 100})


class ReadVideoEndpointTests(TestCase):
    def setUp(self):
        self.client = SentinelClient(SENTINEL_HOSTS, SENTINEL_PORT, SENTINEL_MASTER_NAME)

    def tearDown(self):
        self.client.sentinel.get_master().flushdb()

    def test_read_videos(self):
        _ = self.client.set_one_hash("video.312", "start", 10)
        _ = self.client.set_one_hash("video.312", "complete", 10)
        _ = self.client.set_one_hash("video.312", "firstQuartile", 10)
        _ = self.client.set_one_hash("video.312", "midpoint", 10)
        _ = self.client.set_one_hash("video.312", "thirdQuartile", 10)

        data = self.client.get_one_hash("video.312")
        self.assertEqual(data, {"firstQuartile": 10, "start": 10, "midpoint": 10, "complete": 10, "thirdQuartile": 10})

        res, ok = read_videos("312")
        self.assertTrue(ok)
        self.assertEqual(res, {"firstQuartile": 10, "start": 10, "midpoint": 10, "complete": 10, "thirdQuartile": 10})


class ReadVideoAdEndpointTests(TestCase):
    def setUp(self):
        self.client = SentinelClient(SENTINEL_HOSTS, SENTINEL_PORT, SENTINEL_MASTER_NAME)

    def tearDown(self):
        self.client.sentinel.get_master().flushdb()

    def test_read_video_ads(self):
        _ = self.client.set_one_hash("video_ad.312", "start", 10)
        _ = self.client.set_one_hash("video_ad.312", "complete", 10)
        _ = self.client.set_one_hash("video_ad.312", "firstQuartile", 10)
        _ = self.client.set_one_hash("video_ad.312", "midpoint", 10)
        _ = self.client.set_one_hash("video_ad.312", "thirdQuartile", 10)

        data = self.client.get_one_hash("video_ad.312")
        self.assertEqual(data, {"firstQuartile": 10, "start": 10, "midpoint": 10, "complete": 10, "thirdQuartile": 10})

        res, ok = read_video_ads("312")
        self.assertTrue(ok)
        self.assertEqual(res, {"firstQuartile": 10, "start": 10, "midpoint": 10, "complete": 10, "thirdQuartile": 10})


class UpdateChannelsTests(TestCase):
    def setUp(self):
        self.client = SentinelClient(SENTINEL_HOSTS, SENTINEL_PORT, SENTINEL_MASTER_NAME)

    def tearDown(self):
        self.client.sentinel.get_master().flushdb()

    def test_update_channels(self):
        page_views = Counter({"theonion": 100})
        _ = update_channels(page_views)

        t = date.today()
        today = datetime(t.year, t.month, t.day).strftime("%Y%m%d")
        key = "theonion{}".format(today)
        data = self.client.get_one(key)
        self.assertEqual(data, 100)


class UpdateVideosTests(TestCase):
    def setUp(self):
        self.client = SentinelClient(SENTINEL_HOSTS, SENTINEL_PORT, SENTINEL_MASTER_NAME)

    def tearDown(self):
        self.client.sentinel.get_master().flushdb()

    def test_update_videos(self):
        videos = Counter({
            ("312", "start"): 10,
            ("312", "complete"): 10,
            ("312", "firstQuartile"): 10,
            ("312", "midpoint"): 10,
            ("312", "thirdQuartile"): 10,
        })
        _ = update_videos(videos)

        data = self.client.get_one_hash("video.312")
        self.assertEqual(data, {"firstQuartile": 10, "start": 10, "midpoint": 10, "complete": 10, "thirdQuartile": 10})


class UpdateVideoAdsTests(TestCase):
    def setUp(self):
        self.client = SentinelClient(SENTINEL_HOSTS, SENTINEL_PORT, SENTINEL_MASTER_NAME)

    def tearDown(self):
        self.client.sentinel.get_master().flushdb()

    def test_update_videos(self):
        ads = Counter({
            ("312", "start"): 10,
            ("312", "complete"): 10,
            ("312", "firstQuartile"): 10,
            ("312", "midpoint"): 10,
            ("312", "thirdQuartile"): 10,
        })
        _ = update_video_ads(ads)

        data = self.client.get_one_hash("video_ad.312")
        self.assertEqual(data, {"firstQuartile": 10, "start": 10, "midpoint": 10, "complete": 10, "thirdQuartile": 10})



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from redis.sentinel import Sentinel


class SentinelConnection(object):
    """wrapper around a redis sentinel
    """

    def __init__(self, hosts, port=26379, master_name="mymaster"):
        """sets a sentinel connection and stores a master instance name

        :param hosts: a list of ip addresses
        :type hosts: list

        :param port: the port number to connect to the sentinels
        :type port: int

        :param master_name: the name of the master database
        :type master_name: str
        """
        sentinels = [(host, port) for host in hosts]
        self.sentinel = Sentinel(sentinels)
        self.master_name = master_name

    def get_master(self):
        """gets a connection to the master redis instance

        :return: the master instance of the sentinel cluster
        :rtype: redis.StrictRedis
        """
        return self.sentinel.master_for(self.master_name)

    def get_slave(self):
        """gets a connection to one of the slave redis instances

        :return: a slave instance of the sentinel cluster
        :rtype: redis.StrictRedis
        """
        return self.sentinel.slave_for(self.master_name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-


# sentinels stuff

DEF_SENTINEL_HOSTS = "localhost"
DEF_SENTINEL_PORT = 26379
DEF_SENTINEL_MASTER_NAME = "mymaster"


# strict clients stuff

DEF_REDIS_HOST = "localhost"
DEF_REDIS_PORT = 6379
DEF_REDIS_DB = 0


# pageview stuff

DEF_WRITE_CHANNEL_REGEX_PATTERN = "^(theonion|avclub|clickhole|onionstudios)\\d{8}$"
DEF_WRITE_DATE_REGEX_PATTERN = "\\d{8}"
DEF_READ_CHANNEL_REGEX_PATTERN = "^(theonion|avclub|clickhole|onionstudios)\\d{6,}\\*?"
DEF_READ_DATE_REGEX_PATTERN = "(\\d{6}\\*|\\d{8})"
DEF_DATE_EXTRACTION_REGEX_PATTERN = "\\d{8}"


# video stuff

DEF_VIDEO_PREFIX = "video"
DEF_VIDEO_ID_REGEX_PATTERN = "\\d+"
DEF_VIDEO_EVENT_REGEX_PATTERN = "^(start|complete|firstQuartile|midpoint|thirdQuartile)$"


# video ad stuff

DEF_VIDEO_AD_PREFIX = "video_ad"
DEF_VIDEO_AD_ID_REGEX_PATTERN = "\\d+"
DEF_VIDEO_AD_EVENT_REGEX_PATTERN = "^(start|complete|firstQuartile|midpoint|thirdQuartile)$"


# wsgi stuff

DEF_REFERER_ANCHORED_REGEX_PATTERN = "(staff|www)\\.(theonion|avclub|clickhole|onionstudios)\\.com"
DEF_REFERER_EXTRACT_REGEX_PATTERN = "(theonion|avclub|clickhole|onionstudios)"
DEF_FLUSH_INTERVAL = 10
